OPTIONS PS=55 LS=175 ERRORABEND NOFMTERR ERRORS=1 MPRINT SYMBOLGEN NOCENTER NODATE NONUMBER
;

TITLE1 "SGF2016 - Session 8300 - Heart Codebook";
FOOTNOTE1 "%SYSFUNC(GETOPTION(SYSIN)) - &SYSDATE - &SYSTIME - run by &SYSUSERID";
RUN;

OPTIONS FMTSEARCH=(library.heart work);

TITLE2 "HEART";
RUN;

LIBNAME dd '.';
FILENAME odsout ".";
FILENAME out1 ".\heart_include1.txt"; /* for header macro */
FILENAME out2 ".\heart_include2.txt"; /* for missval macro */
FILENAME out2a ".\heart_include2a.txt"; /* for cmissval macro */
FILENAME out3 ".\heart_include3.txt"; /* for detail macros */
FILENAME out4 ".\heart_include4.txt"; /* for printing macros */
LIBNAME library '.';
RUN;

     /* step 1 - import the metadata spreadsheet for the file to be documented */
     /* although it was created in SAS, it may have been modified to correct identify variable types, etc. */

PROC IMPORT DBMS=EXCEL OUT = dd.heart_cb
    DATAFILE = ".\heart_db.xlsx" REPLACE;
RUN;

     /* step 2 - create some reusable work data sets for codebook */

DATA bottomline;

    blurb='__________________________________________________________________';

    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */

RUN;

     /* step 3 - create macro to generate header information */


%MACRO header(varname,order);

DATA temp&order.a;
    LENGTH sentence1 sentence2 sentence3 $ 500 blurb $ 25000 namecolon labelfmt typec vartypec dsc $ 250;
    SET dd.heart_cb (WHERE=(name="&varname"));
    
    /* we want to make a long sentence with long breaks */

    /* the first sentence is variable name and variable label */
    
    namecolon=CATS('^{STYLE [FONTSIZE=10pt FONTWEIGHT=bold]',name,":}"); 
    labelfmt=CATS('^{STYLE [FONTSIZE=10pt FONTWEIGHT=bold]',label,"}"); 
    sentence1=CATX(' ',namecolon,labelfmt);

    /* the second sentence is variable type */

    IF type=1 THEN typec='Numeric';
    ELSE IF type=2 THEN typec='Character';
    
    vartypec='^{NEWLINE 2}^{NBSPACE 2}^{style [FONTSIZE=10pt FONTSTYLE=italic]Variable Type:}';
    sentence2=CATX(' ',vartypec,typec);

    /* the third sentence is Data File */

    dsc='^{NEWLINE 1}^{NBSPACE 2}^{STYLE [FONTSIZE=10pt FONTSTYLE=italic]Data File:}';

    sentence3=CATX(' ',dsc,"Heart");;

    /* now we want to string into a paragraph for printing; */


    blurb=CATT(sentence1,sentence2,sentence3);
 
    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */

RUN;


DATA header&order;
    SET temp&order.a (KEEP=blurb);
    order=&order;
RUN;

PROC PRINT DATA=header&order;
RUN;


%MEND;

    /* step 4 - create macro to generate missing value information (numeric variables) */


%MACRO missval(varname,order);

DATA miss;
    SET dd.heart (WHERE=(&varname in(.)));
    counter=1;
RUN;

PROC MEANS DATA=miss NOPRINT;
    VAR counter;
    OUTPUT OUT=sum SUM=nmiss;
RUN;


PROC MEANS DATA=dd.heart NOPRINT;
    VAR &varname;
    OUTPUT OUT=temp&order.z N=n ;
RUN;

DATA missval&order (KEEP=blurb order);
    LENGTH sentence1 sentence2  $ 500 
    blurb $ 25000 ncolon nmisscolon  $ 200;
    MERGE temp&order.z sum (KEEP=nmiss);

    order=&order;

    IF nmiss=. then nmiss=0;
    
    ncolon='^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Non-missing values:';
    nmisscolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Missing values:';
  
    putn=PUT(n,comma9.);
    putnmiss=PUT(nmiss,comma9.);
    
    sentence1=CATX(' ',ncolon,putn);
    sentence2=CATX(' ',nmisscolon,putnmiss);

    
    /* now we want to string into a paragraph for printing; */


    blurb=CATT(sentence1,sentence2);
 
    label BLURB ="";  /* this is NOT blank - it is a non-printing character */


RUN;

%MEND;

    /* step 4a - create macro to generate missing value information (character variables) */


%MACRO cmissval(varname,order);

DATA appmiss;
    SET dd.heart (WHERE=(&varname IN(' ')));
    counter=1;
RUN;

DATA present;
    SET dd.heart (WHERE=(&varname NOT IN(' ')));
    counter=1;
RUN;


DATA temp&order.gc (KEEP=length);
    SET dd.heart_cb (WHERE=(name="&varname"));
RUN;


PROC MEANS DATA=appmiss NOPRINT;
    VAR counter;
    OUTPUT OUT=appsum SUM=nmiss;
RUN;

PROC MEANS DATA=present NOPRINT;
    VAR counter;
    OUTPUT OUT=temp&order.z SUM=n ;
RUN;

DATA missval&order (keep=blurb order);
    LENGTH sentence1 sentence2 sentence3 sentence4 $ 500 
    blurb $ 25000 ncolon nmisscolon inappmisscolon lengthcolon $ 200;
    MERGE temp&order.z appsum (KEEP=nmiss) temp&order.gc (KEEP=length);

    order=&order;

    IF nmiss=. THEN nmiss=0;
    IF n=. THEN n=0;

    ncolon='^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Non-missing values:';
    nmisscolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Missing values:';
    lengthcolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Length:';
   
    putn=PUT(n,comma9.);
    putnmiss=PUT(nmiss,comma9.);
    putlength=PUT(length,comma9.);
    
    sentence1=CATX(' ',ncolon,putn);
    sentence2=CATX(' ',nmisscolon,putnmiss);
    sentence3=CATX(' ',lengthcolon,putlength);

    blurb=CATT(sentence1,sentence2,sentence3);
 
    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */


RUN;

%MEND;

    /* step 5 - create a series of macros for 3 different variable types to generate detail information */


%MACRO detailcont(varname,order);

PROC MEANS DATA=dd.heart NOPRINT;
    VAR &varname;
    OUTPUT OUT=temp&order.f MIN=min MAX=max MEAN=mean Q1=q1 MEDIAN=median Q3=q3;
RUN;

DATA detail&order (KEEP=blurb order);
    LENGTH sentence4 sentence5 sentence6 sentence7 sentence8 sentence9 $ 500 
    blurb $ 25000 mincolon maxcolon meancolon q1colon mediancolon q3colon $ 200;
    SET temp&order.f ;

    order=&order;

 
    mincolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Minimum:';
    maxcolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Maximum:';
    meancolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}Mean:';
    q1colon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}25th percentile:';
    mediancolon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}50th percentile:';
    q3colon='^{NEWLINE 1}^{NBSPACE 5}^{Unicode 2022}^{NBSPACE 2}75th percentile:';

    putmin=PUT(min,9.);
    putmax=PUT(max,9.);
    putmean=PUT(mean,comma9.1);
    putq1=PUT(q1,comma9.1);
    putmedian=PUT(median,comma9.1);
    putq3=PUT(q3,comma9.1);

    sentence4=CATX(' ',mincolon,putmin);
    sentence5=CATX(' ',maxcolon,putmax);
    sentence6=CATX(' ',meancolon,putmean);
    sentence7=CATX(' ',q1colon,putq1);
    sentence8=CATX(' ',mediancolon,putmedian);
    sentence9=CATX(' ',q3colon,putq3);

    /* now we want to string into a paragraph for printing; */

    blurb=CATT(sentence4,sentence5,sentence6,sentence7,sentence8,sentence9);
 
    label BLURB ="";  /* this is NOT blank - it is a non-printing character */


RUN;

DATA bottomline;
    LENGTH blurb $ 25000;

    /* now we want to string into a paragraph for printing; */

    blurb='__________________________________________________________________';
    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */
RUN;


DATA print&order;
    LENGTH blurb $ 25000;
    SET header&order 
          missval&order 
          detail&order 
          bottomline;


    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */

RUN;

%MEND;

    
%macro detailcat(varname,order);

DATA print&order;
    LENGTH blurb $ 25000;
    SET header&order 
          missval&order 
          ;

    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */

RUN;

/* get format to be used from contents db and turn into macro variable */

DATA _null_;
    length formata $ 50;
    SET dd.heart_cb (KEEP=format name WHERE=(name="&varname"));
    formata=STRIP(format);
    CALL SYMPUT('fmt',STRIP(formata));
RUN;

/* strip formats so we can get unformatted values */

DATA temp&order.c;
    SET dd.heart ;
    FORMAT _all_;
    INFORMAT _all_;
    IF &varname NOT IN(.,.D,.S,.R,.Z,.U,.N,.C) THEN counter=1;
RUN;

PROC FREQ DATA=temp&order.c (WHERE=(&varname NOT IN(.,.D,.S,.R,.Z,.U,.N,.C)));
    TABLES &varname / NOPRINT OUT=temp&order.d;
RUN;

PROC MEANS DATA=temp&order.c NOPRINT;
    VAR counter;
    OUTPUT OUT=temp&order.ca N=total;
RUN;

DATA temp&order.e (KEEP=unfmtval fmtval frequency pct altshade);
    LENGTH unfmtval frequency pct $ 15 fmtval $ 200;
    SET temp&order.d;

    IF MOD(_n_,2)=0 THEN altshade=1;
    
    unfmtval=PUT(&varname,comma7.);
    fmtval=PUT(&varname,&fmt..);
    frequency=PUT(count,comma9.);
    pct=CATS(PUT(percent,comma7.1),'%');

    LABEL unfmtval='Value'
          fmtval='Label'
          frequency='Frequency'
          pct='%';
RUN;

DATA temp&order.ea (KEEP=unfmtval fmtval frequency pct boldit);
    LENGTH unfmtval frequency pct $ 15 fmtval $ 200;
    SET temp&order.ca;
  
    unfmtval='Total';
    frequency=put(total,comma9.);
    pct='100%';

    boldit=1;

RUN;

DATA detailtab&order.;
    SET temp&order.e temp&order.ea;
    order=&order;
RUN;


%MEND;



    
%MACRO detailcharcat(varname,order);

DATA print&order;
    LENGTH blurb $ 25000;
    SET header&order 
          missval&order 
          ;

    LABEL blurb ="";  /* this is NOT blank - it is a non-printing character */

RUN;

/* First remove any missing values */

DATA temp&order.c;
    SET dd.heart (WHERE=(&varname NOT IN('Logical Skip','Refused','Don''t Know','Question Not Answered',
                                            'Item Non-Response','Not Specified')));
    counter=1;
RUN;

PROC FREQ DATA=temp&order.c ;
    TABLES &varname / NOPRINT out=temp&order.d;
RUN;

PROC MEANS data=temp&order.c NOPRINT;
    VAR counter;
    OUTPUT OUT=temp&order.ca N=total;
RUN;

DATA temp&order.e (KEEP=fmtval frequency pct altshade);
    LENGTH frequency pct $ 15 fmtval $ 200;
    SET temp&order.d;

    IF MOD(_n_,2)=0 THEN altshade=1;
    
    fmtval=&varname.;
    frequency=PUT(count,comma9.);
    pct=CATS(PUT(percent,comma7.1),'%');

    LABEL fmtval='Value'
          frequency='Frequency'
          pct='%';
RUN;

DATA temp&order.ea (KEEP=fmtval frequency pct boldit);
    LENGTH frequency pct $ 15 fmtval $ 200;
    SET temp&order.ca;
  
    fmtval='Total';
    frequency=PUT(total,comma9.);
    pct='100%';

    boldit=1;

RUN;

DATA detailtab&order.;
    SET temp&order.e temp&order.ea;
    order=&order;
RUN;


%MEND;


    /* step 6 - write out files to run macros */

DATA _null_;
    FILE out1 LRECL=80 PAD;
    LENGTH include_string $ 80;
    SET dd.heart_cb (KEEP=varnum name vartype);

    include_string=CATS('%header(',name,",",varnum,");");
    PUT include_string;
RUN;

DATA _null_;
    FILE out2 LRECL=80 PAD;
    LENGTH include_string $ 80;
    SET dd.heart_cb (KEEP=varnum name type WHERE=(type NOT IN(2)));

    include_string=CATS('%missval(',name,",",varnum,");");
    PUT include_string;
RUN;

DATA _null_;
    FILE out2a LRECL=80 PAD;
    LENGTH include_string $ 80;
    SET dd.heart_cb (keep=varnum name type WHERE=(type IN(2)));

    include_string=CATS('%cmissval(',name,",",varnum,");");
    PUT include_string;
RUN;

data _null_;
    FILE out3 LRECL=80 PAD;
    LENGTH include_string $ 80;
    SET dd.heart_cb (keep=varnum name vartype);

    IF vartype=1 THEN include_string=CATS('%detailcat(',name,",",varnum,");");
    IF vartype=2 THEN include_string=CATS('%detailcharcat(',name,",",varnum,");");
    IF vartype=3 THEN include_string=CATS('%detailcont(',name,",",varnum,");");

    PUT include_string;
RUN;
    
DATA _null_;
    FILE out4 LRECL=80 PAD;
    LENGTH include_string $ 80;
    SET dd.heart_cb (KEEP=varnum name vartype);


    IF vartype=1 THEN include_string=CATS('%printtable(',varnum,");");
    IF vartype=2 THEN include_string=CATS('%printtablec(',varnum,");");
    IF vartype=3 THEN include_string=CATS('%printblurb(',varnum,");");

    PUT include_string;
RUN;

    /* step 8 - run the codebook macros in order (1,2,3) */

    %INCLUDE ".\heart_include1.txt";
    %INCLUDE ".\heart_include2.txt";
    %INCLUDE ".\heart_include2a.txt";
    %INCLUDE ".\heart_include3.txt";

    /* step 9 - create printing macros */

  
%MACRO printblurb(order);

ODS TAGSETS.RTF STYLE=styles.noborder;
ODS STARTPAGE=no;

PROC REPORT NOWD DATA=print&order 
    STYLE(report)=[cellpadding=3pt vjust=b]
    STYLE(header)=[just=center font_face=Helvetica font_weight=bold font_size=10pt]
    STYLE(lines)=[just=left font_face=Helvetica] ;
  COLUMNS blurb ;
  DEFINE blurb / style(COLUMN)={just=l font_face=Helvetica 
           font_size=10pt cellwidth=988 }
                      style(HEADER)={just=l font_face=Helvetica 
           font_size=10pt };
RUN;

ODS STARTPAGE=no;

%MEND;

%MACRO printtable(order);

ODS TAGSETS.RTF STYLE=styles.noborder;
ODS STARTPAGE=no;

proc report nowd data=print&order 
    STYLE(report)=[cellpadding=3pt vjust=b]
    STYLE(header)=[just=center font_face=Helvetica font_weight=bold font_size=10pt]
    STYLE(lines)=[just=left font_face=Helvetica] ;
  COLUMNS blurb ;
  DEFINE blurb / style(COLUMN)={just=l font_face=Helvetica 
           font_size=10pt cellwidth=988 }
                      style(HEADER)={just=l font_face=Helvetica 
           font_size=10pt };
RUN;

ODS TEXT ='^n ';


ODS TAGSETS.RTF STYLE=styles.minimal;
ODS STARTPAGE=no;

proc report nowd data=detailtab&order /* spacing=8 */
    STYLE(report)=[cellpadding=3pt vjust=b]
    STYLE(header)=[just=center font_face=Helvetica font_weight=bold font_size=10pt background=graydd
                   borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white]
    STYLE(lines)=[just=left font_face=Helvetica] split='|';
  COLUMNS unfmtval fmtval frequency pct boldit altshade;
  DEFINE boldit / display ' ' noprint;
  DEFINE altshade / display ' ' noprint;
  DEFINE unfmtval / style(COLUMN)={just=c font_face=Helvetica foreground=black 
           font_size=10pt cellwidth=170 borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} CENTER
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold  
           font_size=10pt  };
  DEFINE fmtval / style(COLUMN)={just=l font_face=Helvetica foreground=black 
           font_size=10pt cellwidth=440  borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} LEFT
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold   
           font_size=10pt  };
  DEFINE frequency / style(COLUMN)={just=c font_face=Helvetica foreground=black 
           font_size=10pt cellwidth=170  borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} CENTER
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold 
           font_size=10pt  };
  DEFINE pct / style(COLUMN)={just=c font_face=Helvetica foreground=black
           font_size=10pt cellwidth=170  borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} CENTER
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold   
           font_size=10pt  };
COMPUTE boldit;
  IF (boldit EQ 1) THEN CALL DEFINE(_row_,"STYLE","STYLE=[FONT_WEIGHT=BOLD BACKGROUND=GRAYDD]");
  ENDCOMP;

COMPUTE altshade;
  IF (altshade EQ 1) THEN CALL DEFINE(_row_,"STYLE","STYLE=[BACKGROUND=GRAYEE]");
  ENDCOMP;
 
RUN;

ODS STARTPAGE=no;

%MEND;

%macro printtablec(order);

ODS TAGSETS.RTF STYLE=styles.noborder;
ods STARTPAGE=no;

proc report nowd data=print&order 
    STYLE(report)=[cellpadding=3pt vjust=b]
    STYLE(header)=[just=center font_face=Helvetica font_weight=bold font_size=10pt]
    STYLE(lines)=[just=left font_face=Helvetica] ;
  COLUMNS blurb ;
  DEFINE blurb / style(COLUMN)={just=l font_face=Helvetica 
           font_size=10pt cellwidth=988 }
                      style(HEADER)={just=l font_face=Helvetica 
           font_size=10pt };
RUN;

ODS TEXT='^n ';


ODS TAGSETS.RTF STYLE=styles.minimal;
ods STARTPAGE=no;

proc report nowd data=detailtab&order /* spacing=8 */
    STYLE(report)=[cellpadding=3pt vjust=b]
    STYLE(header)=[just=center font_face=Helvetica font_weight=bold font_size=10pt background=graydd
                   borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white]
    STYLE(lines)=[just=left font_face=Helvetica] split='|';
  COLUMNS fmtval frequency pct boldit altshade;
  DEFINE boldit / display ' ' noprint;
  DEFINE altshade / display ' ' noprint;
  DEFINE fmtval / style(COLUMN)={just=l font_face=Helvetica foreground=black 
           font_size=10pt cellwidth=610  borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} LEFT
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold   
           font_size=10pt  };
  DEFINE frequency / style(COLUMN)={just=c font_face=Helvetica foreground=black 
           font_size=10pt cellwidth=170  borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} CENTER
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold 
           font_size=10pt  };
  DEFINE pct / style(COLUMN)={just=c font_face=Helvetica foreground=black
           font_size=10pt cellwidth=170  borderrightcolor=white borderleftcolor=white bordertopcolor=white borderbottomcolor=white} CENTER
                      style(HEADER)={just=c font_face=Helvetica font_weight=bold   
           font_size=10pt  };
COMPUTE boldit;
  IF (boldit EQ 1) THEN CALL DEFINE(_row_,"STYLE","STYLE=[FONT_WEIGHT=BOLD BACKGROUND=GRAYDD]");
  ENDCOMP;

COMPUTE altshade;
  IF (altshade EQ 1) THEN CALL DEFINE(_row_,"STYLE","STYLE=[BACKGROUND=GRAYEE]");
  ENDCOMP;
 
RUN;

ODS STARTPAGE=no;

%MEND;

    /* step 10 - run the printing macro INSIDE the TAGSETS RTF call */

ODS LISTING CLOSE;
TITLE1;
TITLE2;
FOOTNOTE1;

OPTIONS TOPMARGIN=1in LEFTMARGIN=1in RIGHTMARGIN=1in BOTTOMMARGIN=1in PAPERSIZE=letter;

ODS TAGSETS.RTF FILE="heart_codebook.rtf" OPTIONS(vspace='no') PATH=odsout STYLE=styles.noborder;


ODS TEXT="Heart Data File Codebook";

ODS ESCAPECHAR='^';

    %INCLUDE ".\heart_include4.txt";

ODS TAGSETS.RTF CLOSE;


ENDSAS;



