OPTIONS PS=55 LS=175 ERRORABEND NOFMTERR ERRORS=1 MPRINT SYMBOLGEN NOCENTER;

TITLE1 "SAS GLOBAL FORUM 2016 Session 8300";
FOOTNOTE1 "%SYSFUNC(GETOPTION(SYSIN)) - &SYSDATE - &SYSTIME - run by &SYSUSERID";
RUN;


LIBNAME dd '.';
LIBNAME library '.';
FILENAME odsout '.';
RUN;


%LET dsname=HEART;


ODS RTF FILE='sashelpheart_CONTENTS.RTF' PATH=odsout STYLE=styles.journal2;

PROC CONTENTS DATA=sashelp.heart;
RUN;

PROC PRINT DATA=sashelp.heart (OBS=5) NOOBS;
RUN;

ODS RTF CLOSE;


PROC FORMAT ;
    VALUE startage 25 - 34='25 to 34 years'
                   35 - 44='35 to 44 years'
                   45 - 54='45 to 55 years'
                   55 - 64='55 to 64 years';

    VALUE agefmt 1='25 to 34 years'
                   2='35 to 44 years'
                   3='45 to 54 years'
                   4='55 to 64 years';
RUN;


DATA dd.heart (LABEL="Copy of SASHELP.HEART for SAS GLOBAL FORUM 2016 Session 8300 - created by %SYSFUNC(GETOPTION(SYSIN))
 - &SYSDATE - &SYSTIME - run by &SYSUSERID");
    LENGTH dslabel $ 200 source $ 32;
    SET sashelp.heart;

    /* put in some missing labels */

    dslabel="Copy of SASHELP.HEART for SAS GLOBAL FORUM 2016 Session 8300 - created by %SYSFUNC(GETOPTION(SYSIN))
 - &SYSDATE - &SYSTIME - RUN by &SYSUSERID";
    source="&dsname";

    IF 25 LE ageatstart LE 34 THEN age=1;
    IF 35 LE ageatstart LE 44 THEN age=2;
    IF 45 LE ageatstart LE 54 THEN age=3;
    IF 55 LE ageatstart LE 64 THEN age=4;

    IF ageatstart ge 85 THEN age=7;
    FORMAT age agefmt.;

    LABEL choLEsterol='Cholesterol level'
          diastolic='Diastolic blood pressure'
          height='Height'
          sex='Gender'
          smoking='Cigarettes per day'
          status='Wanted, dead or alive'
          systolic='Systolic blood pressure'
          weight='Weight'
          source='Data set name'
          dslabel='Data set information'
          age='Age at Start Category'
          ;

RUN;

PROC CONTENTS DATA=dd.heart OUT=heartconts;
RUN;

PROC PRINT DATA=heartconts (OBS=5) NOOBS;
RUN;

PROC FREQ DATA=dd.heart;
    TABLES age*ageatstart / MISSING LIST;
RUN;

DATA heartcontsa;
    SET heartconts (KEEP=name format label length name npos type varnum memname memlabel);

    IF type=1 AND format NE '' THEN vartype=1;
    IF type=1 AND format EQ '' THEN vartype=3;
    IF type=2 THEN vartype=2;

    source=memname;
    dsinfo=memlabel;
    DROP mem: ;

RUN;

DATA dd.heart_cb;
    RETAIN varnum vartype name label FORMAT LEngth npos type source dsinfo;
    SET heartcontsa;
RUN;

PROC SORT DATA=dd.heart_cb;
    BY varnum;
RUN;

PROC EXPORT DATA = dd.heart_cb DBMS = excel 
             OUTFILE = ".\heart_db.xlsx" REPLACE;
RUN;

/* if running interactively comment endsas out */
ENDSAS;

